unit projstat_statisticaldata;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, LazIDEIntf, ProjectIntf, SrcEditorIntf,
  PropEdits, FormEditingIntf, LMessages, projstat_timer;

type
   TUserAction = (uaEditing, uaDesigning, uaOther);

  { TStatisticalData }

  TStatisticalData = class
  private
    FCurProject: TLazProject;
    FThreadTimer: TThreadTimer;
    FUserAction: TUserAction;
    function OnProjectOpened(Sender: TObject; AProject: TLazProject): TModalResult;
    function OnProjectClose(Sender: TObject; {%H-}AProject: TLazProject): TModalResult;
    procedure OnLogStatistics(Sender: TObject);
    procedure OnIdle(Sender: TObject; var {%H-}Done: Boolean);
    procedure OnUserInput(Sender: TObject; {%H-}Msg: Cardinal);
    procedure OnEditorWindowFocused(Sender: TObject);
    procedure OnEditorActivate(Sender: TObject);
    procedure OnActiveFormChanged(Sender: TObject; Form: TCustomForm);
    procedure SetUserAction;
    function IsMainFormAvailable: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
  end;

var
  StatisticalData: TStatisticalData;

implementation

uses projstat_mainform, projstat_const;
{ TStatisticalData }

function TStatisticalData.OnProjectOpened(Sender: TObject; AProject: TLazProject): TModalResult;
begin
  Result := mrOk;
  FCurProject := AProject;
  if Assigned(FThreadTimer) then
    FThreadTimer.Enabled := True;
  if (FCurProject <> nil) and (IsMainFormAvailable) then
  begin
    MainForm.lbProjName.Caption := ExtractFileName(FCurProject.MainFile.GetFullFilename);
    MainForm.lbProjPath.Caption := FCurProject.MainFile.GetFullFilename;
    MainForm.lbProjFiles.Caption := IntToStr(FCurProject.FileCount);
  end;
end;

function TStatisticalData.OnProjectClose(Sender: TObject; AProject: TLazProject): TModalResult;
begin
  Result := mrOk;
  FCurProject := nil;
  FThreadTimer.Enabled := False;
end;

procedure TStatisticalData.OnLogStatistics(Sender: TObject);
begin
  //
end;

procedure TStatisticalData.OnIdle(Sender: TObject; var Done: Boolean);
begin
  FThreadTimer.Idle := True;
  if IsMainFormAvailable then
    if not FThreadTimer.CanLog then
      MainForm.lbActiveWindowUserStatus.Caption := 'Idle';
end;

procedure TStatisticalData.OnUserInput(Sender: TObject; Msg: Cardinal);
begin
  if (Msg = LM_NCMOUSEMOVE) or (Msg = LM_MOUSEMOVE) or (Msg = 66637) then
    Exit;
  FThreadTimer.UserInput := True;
  if IsMainFormAvailable then
    MainForm.lbActiveWindowUserStatus.Caption := 'Active';
end;

procedure TStatisticalData.OnEditorActivate(Sender: TObject);
begin
  if IsMainFormAvailable then
  begin
    if SourceEditorManagerIntf.ActiveEditor <> nil then
      MainForm.lbProjActiveFile.Caption := ExtractFileName(SourceEditorManagerIntf.ActiveEditor.FileName)
    else
      MainForm.lbProjActiveFile.Caption := '';
    SetUserAction;
  end;
end;

procedure TStatisticalData.OnEditorWindowFocused(Sender: TObject);
begin
  if SourceEditorManagerIntf.ActiveEditor <> nil then
  begin
    FUserAction := uaEditing;
    SetUserAction;
  end;
end;

procedure TStatisticalData.OnActiveFormChanged(Sender: TObject; Form: TCustomForm);
begin
  if Form = nil then
    Exit;

 //Lazarus main form, don't change the status
  if Form.Name <> rsStatisticalData_MainForm then
    FUserAction := uaOther;

  if IsFormDesign(Form) or (Form.Name = rsStatisticalData_ObjectInspector) then
    FUserAction := uaDesigning;

  if IsMainFormAvailable then
  begin
    if Form <> nil then
    begin
      MainForm.lbActiveWindowName.Caption := Form.Name;
      MainForm.lbActiveWindowCaption.Caption := Form.Caption;
    end
    else
    begin
      MainForm.lbActiveWindowName.Caption := '';
      MainForm.lbActiveWindowCaption.Caption := '';
    end;
    SetUserAction;
  end
end;

procedure TStatisticalData.SetUserAction;
begin
  case FUserAction of
    uaDesigning: MainForm.lbActiveWindowUserAction.Caption := 'Designing';
    uaEditing: MainForm.lbActiveWindowUserAction.Caption := 'Editing';
    uaOther: MainForm.lbActiveWindowUserAction.Caption := 'Other';
  end;
end;

function TStatisticalData.IsMainFormAvailable: Boolean;
begin
  Result := Assigned(MainForm) and not LazarusIDE.IDEIsClosing;
end;

constructor TStatisticalData.Create;
begin
  LazarusIDE.AddHandlerOnProjectOpened(@OnProjectOpened);
  LazarusIDE.AddHandlerOnProjectClose(@OnProjectClose);
  Application.AddOnIdleHandler(@OnIdle, True);
  Application.AddOnUserInputHandler(@OnUserInput, True);
  SourceEditorManagerIntf.RegisterChangeEvent(semWindowFocused, @OnEditorWindowFocused);
  SourceEditorManagerIntf.RegisterChangeEvent(semEditorActivate, @OnEditorActivate);
  Screen.AddHandlerActiveFormChanged(@OnActiveFormChanged);
  FThreadTimer := TThreadTimer.Create(True);
  FThreadTimer.OnLogStatistics := @OnLogStatistics;
  FThreadTimer.Start
end;

destructor TStatisticalData.Destroy;
begin
  FThreadTimer.Enabled := False;
  FThreadTimer.Terminate;
  Application.RemoveOnIdleHandler(@OnIdle);
  Application.RemoveOnUserInputHandler(@OnUserInput);
  inherited destroy;
end;

end.

