unit projstat_timer;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LCLIntf;

type

  { TThreadTimer }

  TThreadTimer = class(TThread)
  private
    FTime: QWord;
    FIdleTime: QWord;
    FInterval: Cardinal;
    FOnLogStatistics: TNotifyEvent;
    FEnabled: Boolean;
    FCanLog: Boolean;
    FIdle: Boolean;
    FIdleInterval: Cardinal;
    FIdleStarted: Boolean;
    FUserInput: Boolean;
    procedure DoOnLogStatistics;
    procedure SetIdle(AValue: Boolean);
    procedure SetUserInput(AValue: Boolean);
  protected
    procedure Execute; override;
  public
    property OnLogStatistics: TNotifyEvent read FOnLogStatistics write FOnLogStatistics;
    property Enabled: Boolean read FEnabled write FEnabled;
    property IdleInterval: Cardinal read FIdleInterval write FIdleInterval;
    property Idle: Boolean read FIdle write SetIdle;
    property UserInput: Boolean read FUserInput write SetUserInput;
    property CanLog: Boolean read FCanLog;
  public
    constructor Create(CreateSuspended: Boolean);
    destructor Destroy; override;
  end;

implementation

constructor TThreadTimer.Create(CreateSuspended: Boolean);
begin
  inherited Create(CreateSuspended);
  FInterval := 1000;
  FIdleInterval := 2000;
  FreeOnTerminate := True;
  FEnabled := False;
  FTime := GetTickCount64;
  FIdleTime := GetTickCount64;
end;

destructor TThreadTimer.Destroy;
begin
  inherited Destroy;
end;

procedure TThreadTimer.DoOnLogStatistics;
begin
  if Assigned(FOnLogStatistics) then
    FOnLogStatistics(Self);
end;

procedure TThreadTimer.SetIdle(AValue: Boolean);
begin
  if AValue <> FIdle then
    FIdle := AValue;
  if (FIdle) and (not FIdleStarted) then
  begin
    FIdleTime := GetTickCount64;
    FIdleStarted := True;
    FUserInput := False;
  end;
end;

procedure TThreadTimer.SetUserInput(AValue: Boolean);
begin
  if AValue <> FUserInput then
    FUserInput := AValue;
  if FUserInput then
  begin
    FIdleStarted := False;
    FCanLog := True;
    FIdle := False;
  end;
end;

procedure TThreadTimer.Execute;
begin
  while not Terminated do
  begin
    Sleep(1);
    if (GetTickCount64 - FTime > FInterval) and (FEnabled) then
    begin
      FTime := GetTickCount64;
      if FIdle then
        if (GetTickCount64 - FIdleTime > FIdleInterval) then
          FCanLog := False;
      if FCanLog then
        Synchronize(@DoOnLogStatistics);
    end;
  end;
end;

end.

